package com.halomobi.download.database;


import static com.halomobi.common.ConstantPool.EroType.INSTALL_FAILED;
import static com.halomobi.download.bean.DownloadItem.STATE_FINISH;
import static com.halomobi.download.bean.DownloadItem.STATE_INSTALLED;
import static com.halomobi.download.database.DownloadDBHelper.ADD_INSTALL_FAILED_INFO;
import static com.halomobi.download.database.DownloadDBHelper.ADD_SQL;
import static com.halomobi.download.database.DownloadDBHelper.APP_NAME;
import static com.halomobi.download.database.DownloadDBHelper.APP_PACKAGE_NAME;
import static com.halomobi.download.database.DownloadDBHelper.CLICK_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.COUNT;
import static com.halomobi.download.database.DownloadDBHelper.DEEP_LINK_PATH;
import static com.halomobi.download.database.DownloadDBHelper.DEEP_LINK_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.DOWNLOADED_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.DOWNLOAD_EXPIRATION_TIME;
import static com.halomobi.download.database.DownloadDBHelper.DOWNLOAD_START_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.E_TAG;
import static com.halomobi.download.database.DownloadDBHelper.FILE_DESC;
import static com.halomobi.download.database.DownloadDBHelper.FILE_DOWN_PATH;
import static com.halomobi.download.database.DownloadDBHelper.FILE_DOWN_URL;
import static com.halomobi.download.database.DownloadDBHelper.FILE_NAME;
import static com.halomobi.download.database.DownloadDBHelper.FILE_SIZE;
import static com.halomobi.download.database.DownloadDBHelper.FILE_STATE;
import static com.halomobi.download.database.DownloadDBHelper.INSTALLED_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.INSTALL_FAILED_TABLE_NAME;
import static com.halomobi.download.database.DownloadDBHelper.INSTALL_START_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.NET_TYPE;
import static com.halomobi.download.database.DownloadDBHelper.OPEN_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.START_TIME;
import static com.halomobi.download.database.DownloadDBHelper.TABLE_NAME;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import com.halomobi.download.bean.DownloadItem;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.LogUtils;

import org.json.JSONStringer;

import java.io.File;
import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicInteger;

public class DownloadDBManager {

    private static DownloadDBManager mInstance;
    private AtomicInteger mOpenCounter = new AtomicInteger();
    private SQLiteOpenHelper mDatabaseHelper;
    private SQLiteDatabase mDatabase;

    private DownloadDBManager(Context context) {
        mDatabaseHelper = new DownloadDBHelper(context);
    }

    public static synchronized DownloadDBManager getInstance(Context context) {
        if (mInstance == null) {
            mInstance = new DownloadDBManager(context);
        }
        return mInstance;
    }

    /**
     * 添加下载任务
     *
     * @param info 下载任务信息
     * @throws Exception
     */
    public synchronized void add(DownloadItem info) throws Exception {
        openDatabase().execSQL(ADD_SQL, new Object[]{info.getName(), info.getCurrentFileSize(),
                info.getDescription(), info.getDownLoadUrl(), info.getDownloadedTracker(),
                info.getInstalledTracker(), info.getOpenTracker(),
                info.getClickTracker(), info.getDeepLinkTracker(), info.getDeepLink(), info.getSavePath(),
                info.getCurrentState(), info.getExpirationTime(), System.currentTimeMillis(),
                info.getNetType(), info.geteTag()});
        closeDatabase();
    }

    /**
     * 查询下载任务，包含删除过期任务机制
     *
     * @return 数据库中的下载任务
     */
    public synchronized ArrayList<DownloadItem> query() {

        SQLiteDatabase db = null;
        try {
            db = openDatabase();
            db.beginTransaction();
            db.delete(TABLE_NAME, " ? - " + START_TIME + " > " + DOWNLOAD_EXPIRATION_TIME,
                    new String[]{System.currentTimeMillis() + ""});
            db.setTransactionSuccessful();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (db != null) {
                if (db.inTransaction()) {
                    db.endTransaction();
                }
                closeDatabase();
            }
        }

        Cursor cursor = null;
        ArrayList<DownloadItem> downloadInfos = new ArrayList<>();
        try {
            db = openDatabase();
            cursor = db.query(TABLE_NAME, new String[]{FILE_DESC, FILE_SIZE,
                            FILE_DOWN_PATH, FILE_DOWN_URL, E_TAG, FILE_STATE, DOWNLOAD_EXPIRATION_TIME,
                            FILE_NAME, DOWNLOAD_START_TRACKER, DOWNLOADED_TRACKER, OPEN_TRACKER, INSTALL_START_TRACKER,
                            INSTALLED_TRACKER, CLICK_TRACKER, DEEP_LINK_TRACKER, DEEP_LINK_PATH, NET_TYPE},
                    null, null, null, null, null);
            if (cursor.moveToFirst()) {
                do {
                    DownloadItem info = new DownloadItem();
                    info.setDescription(cursor.getString(cursor.getColumnIndex(FILE_DESC)));
                    info.setCurrentFileSize(cursor.getInt(cursor.getColumnIndex(FILE_SIZE)));
                    info.setSavePath(cursor.getString(cursor.getColumnIndex(FILE_DOWN_PATH)));
                    info.setDownLoadUrl(cursor.getString(cursor.getColumnIndex(FILE_DOWN_URL)));
                    info.seteTag(cursor.getString(cursor.getColumnIndex(E_TAG)));
                    info.setCurrentState(cursor.getInt(cursor.getColumnIndex(FILE_STATE)));
                    info.setExpirationTime(cursor.getInt(cursor.getColumnIndex(DOWNLOAD_EXPIRATION_TIME)));
                    info.setName(cursor.getString(cursor.getColumnIndex(FILE_NAME)));
                    info.setDownloadStartTrackerTracker(cursor.getString(cursor.getColumnIndex(DOWNLOAD_START_TRACKER)));
                    info.setDownloadedTracker(cursor.getString(cursor.getColumnIndex(DOWNLOADED_TRACKER)));
                    info.setOpenTracker(cursor.getString(cursor.getColumnIndex(OPEN_TRACKER)));
                    info.setDeepLinkTracker(cursor.getString(cursor.getColumnIndex(DEEP_LINK_TRACKER)));
                    info.setDeepLink(cursor.getString(cursor.getColumnIndex(DEEP_LINK_PATH)));
                    info.setInstallStartTracker(cursor.getString(cursor.getColumnIndex(INSTALL_START_TRACKER)));
                    info.setInstalledTracker(cursor.getString(cursor.getColumnIndex(INSTALLED_TRACKER)));
                    info.setClickTracker(cursor.getString(cursor.getColumnIndex(CLICK_TRACKER)));
                    info.setNetType(cursor.getInt(cursor.getColumnIndex(NET_TYPE)));

                    downloadInfos.add(info);
                } while (cursor.moveToNext());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FileUtils.close(cursor);
            closeDatabase();
        }
        return downloadInfos;

    }

    /**
     * 删除下载任务
     *
     * @param info 下载任务信息
     */
    public synchronized void delete(DownloadItem info) {
        if (info == null) {
            return;
        }
        SQLiteDatabase db = null;
        try {
            db = openDatabase();
            db.beginTransaction();

            db.delete(TABLE_NAME, FILE_DOWN_URL + " = ? ",
                    new String[]{info.getDownLoadUrl()});

            db.setTransactionSuccessful();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (db != null) {
                if (db.inTransaction()) {
                    db.endTransaction();
                }
                closeDatabase();
            }
        }

        File file = new File(info.getSavePath());
        if (file.exists()) {
            if (file.delete()) {
                LogUtils.d("安装包成功删除");
            } else {
                LogUtils.d("安装包删除失败");
            }
        }

    }

    /**
     * 更新下载任务信息
     *
     * @param info 下载任务信息
     */
    public synchronized void update(DownloadItem info) {
        if (info == null) {
            return;
        }
        SQLiteDatabase db = null;
        try {
            db = openDatabase();
            db.beginTransaction();


            ContentValues values = new ContentValues();
            values.put(FILE_NAME,info.getName());
            values.put(DOWNLOAD_START_TRACKER, info.getDownloadStartTracker());
            values.put(DOWNLOADED_TRACKER, info.getDownloadedTracker());
            values.put(INSTALL_START_TRACKER, info.getInstallStartTracker());
            values.put(INSTALLED_TRACKER, info.getInstalledTracker());
            values.put(OPEN_TRACKER, info.getOpenTracker());
            values.put(DEEP_LINK_TRACKER, info.getDeepLinkTracker());
            values.put(DEEP_LINK_PATH, info.getDeepLink());
            values.put(CLICK_TRACKER, info.getClickTracker());
            if (info.getCurrentState() == STATE_INSTALLED || info.getCurrentState() == STATE_FINISH) {
                values.put(FILE_STATE, info.getCurrentState());
            }
            values.put(E_TAG, info.geteTag());
            values.put(FILE_SIZE, info.getCurrentFileSize());
            values.put(NET_TYPE, info.getNetType());

            db.update(TABLE_NAME, values, FILE_DOWN_URL + " = ? ",
                    new String[]{info.getDownLoadUrl()});

            db.setTransactionSuccessful();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (db != null) {
                if (db.inTransaction()) {
                    db.endTransaction();
                }
                closeDatabase();
            }
        }
    }

    /**
     * 删除监播（避免重复发送）
     *
     * @param item        下载任务信息
     * @param trackerType 监播类型
     */
    public void deleteTracker(DownloadItem item, String trackerType) {
        if (item == null) {
            return;
        }
        switch (trackerType) {
            case CLICK_TRACKER:
                item.setClickTracker("");
                break;
            case DOWNLOADED_TRACKER:
                item.setDownloadedTracker("");
                break;
            case INSTALLED_TRACKER:
                item.setInstalledTracker("");
                break;
            case OPEN_TRACKER:
                item.setOpenTracker("");
                break;
            case DEEP_LINK_TRACKER:
                item.setDeepLinkTracker("");
                break;
            default:
                break;
        }

        SQLiteDatabase db = null;
        try {
            db = openDatabase();
            db.beginTransaction();

            ContentValues values = new ContentValues();
            values.put(trackerType, "");

            db.update(TABLE_NAME, values, FILE_DOWN_URL + " = ? ",
                    new String[]{item.getDownLoadUrl()});

            db.setTransactionSuccessful();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (db != null) {
                if (db.inTransaction()) {
                    db.endTransaction();
                }
                closeDatabase();
            }
        }
    }

    //-----------------记录安装失败信息--------------------

    /**
     * 记录安装失败信息
     *
     * @param appName        应用名
     * @param appPackageName 应用包名
     */
    public void recordInstallFailedInfo(String appName, String appPackageName) throws Exception {
        SQLiteDatabase database = openDatabase();

        Cursor cursor = database.query(INSTALL_FAILED_TABLE_NAME,
                new String[]{APP_NAME, APP_PACKAGE_NAME, COUNT}, null, null, null, null, null);
        if (cursor == null) {
            return;
        }

        if (cursor.getCount() > 0) {
            database.execSQL("UPDATE " + INSTALL_FAILED_TABLE_NAME + " SET count = count + 1 WHERE " + APP_NAME + " = ? AND " + APP_PACKAGE_NAME + " = ?", new Object[]{appName, appPackageName});
        } else {
            database.execSQL(ADD_INSTALL_FAILED_INFO, new Object[]{appName, appPackageName});
        }
        FileUtils.close(cursor);
        closeDatabase();
    }

    /**
     * 读取安装失败信息
     *
     * @return 失败信息
     */
    public String getInstallFailedInfo() {
        SQLiteDatabase db = openDatabase();
        Cursor cursor = db.query(INSTALL_FAILED_TABLE_NAME,
                new String[]{APP_NAME, APP_PACKAGE_NAME, COUNT}, null, null, null, null, null);
        if (cursor == null) {
            return "";
        }

        int mCount = 0;

        JSONStringer js = null;
        try {
            if (cursor.moveToFirst()) {
                js = new JSONStringer();
                js.object();
                js.key("apps").array();
                do {
                    int count;
                    js.object().key(APP_NAME).value(cursor.getString(cursor.getColumnIndex(APP_NAME)))
                            .key(APP_PACKAGE_NAME).value(cursor.getString(cursor.getColumnIndex(APP_PACKAGE_NAME)))
                            .key(COUNT).value(count = cursor.getInt(cursor.getColumnIndex(COUNT))).endObject();
                    //记录总数
                    mCount += count;
                } while (cursor.moveToNext());
                js.endArray();

                js.key("count").value(mCount).key("ero_code").value(INSTALL_FAILED);
                js.endObject();

                //删除库中数据
                db.delete(INSTALL_FAILED_TABLE_NAME, null, null);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FileUtils.close(cursor);
            closeDatabase();
        }

        return js == null ? null : js.toString();
    }

    //----------------- 创建DB -------------------

    private synchronized SQLiteDatabase openDatabase() {
        if (mOpenCounter.incrementAndGet() == 1) {
            // Opening new database
            mDatabase = mDatabaseHelper.getWritableDatabase();
        }
        return mDatabase;
    }

    private synchronized void closeDatabase() {
        if (mOpenCounter.decrementAndGet() == 0 && mDatabase != null) {
            // Closing database
            mDatabase.close();
        }
    }

}
